@@ -8,7 +8,7 @@
 # No external libraries such as ImageMagick are used here, this is a very lightweight solution to
 # finding image information.
 #
-# FastImage knows about GIF, JPEG, BMP, TIFF, ICO, CUR, PNG, PSD and WEBP files.
+# FastImage knows about GIF, JPEG, BMP, TIFF, ICO, CUR, PNG, PSD, SVG and WEBP files.
 #
 # FastImage can also read files from the local filesystem by supplying the path instead of a uri.
 # In this case FastImage reads the file in chunks of 256 bytes until
@@ -20,6 +20,14 @@
 #
 # FastImage will follow up to 4 HTTP redirects to get the image.
 #
+# FastImage also provides a reader for the content length header provided in HTTP.
+# This may be useful to assess the file size of an image, but do not rely on it exclusively -
+# it will not be present in chunked responses for instance.
+#
+# FastImage accepts additional HTTP headers. This can be used to set a user agent
+# or referrer which some servers require. Pass an :http_header argument to specify headers,
+# e.g., :http_header => {'User-Agent' => 'Fake Browser'}.
+#
 # === Examples
 #   require 'fastimage'
 #
@@ -48,7 +56,7 @@ require 'pathname'
 require 'zlib'
 
 class FastImage
-  attr_reader :size, :type
+  attr_reader :size, :type, :content_length
 
   attr_reader :bytes_read
 
@@ -76,7 +84,7 @@ class FastImage
   # If you wish FastImage to raise if it cannot size the image for any reason, then pass
   # :raise_on_failure => true in the options.
   #
-  # FastImage knows about GIF, JPEG, BMP, TIFF, ICO, CUR, PNG, PSD and WEBP files.
+  # FastImage knows about GIF, JPEG, BMP, TIFF, ICO, CUR, PNG, PSD, SVG and WEBP files.
   #
   # === Example
   #
@@ -158,10 +166,16 @@ class FastImage
   end
 
   def initialize(uri, options={})
-    @property = options[:type_only] ? :type : :size
-    @timeout = options[:timeout] || DefaultTimeout
-    @proxy_url = options[:proxy]
     @uri = uri
+    @options = {
+      :type_only        => false,
+      :timeout          => DefaultTimeout,
+      :raise_on_failure => false,
+      :proxy            => nil,
+      :http_header      => {}
+    }.merge(options)
+
+    @property = @options[:type_only] ? :type : :size
 
     if uri.respond_to?(:read)
       fetch_using_read(uri)
@@ -181,17 +195,17 @@ class FastImage
 
     uri.rewind if uri.respond_to?(:rewind)
 
-    raise SizeNotFound if options[:raise_on_failure] && @property == :size && !@size
+    raise SizeNotFound if @options[:raise_on_failure] && @property == :size && !@size
 
   rescue Timeout::Error, SocketError, Errno::ECONNREFUSED, Errno::EHOSTUNREACH, Errno::ECONNRESET,
     ImageFetchFailure, Net::HTTPBadResponse, EOFError, Errno::ENOENT
-    raise ImageFetchFailure if options[:raise_on_failure]
+    raise ImageFetchFailure if @options[:raise_on_failure]
   rescue NoMethodError  # 1.8.7p248 can raise this due to a net/http bug
-    raise ImageFetchFailure if options[:raise_on_failure]
+    raise ImageFetchFailure if @options[:raise_on_failure]
   rescue UnknownImageType
-    raise UnknownImageType if options[:raise_on_failure]
+    raise UnknownImageType if @options[:raise_on_failure]
   rescue CannotParseImage
-    if options[:raise_on_failure]
+    if @options[:raise_on_failure]
       if @property == :size
         raise SizeNotFound
       else
@@ -210,8 +224,10 @@ class FastImage
   end
 
   def fetch_using_http_from_parsed_uri
+    http_header = {'Accept-Encoding' => 'identity'}.merge(@options[:http_header])
+
     setup_http
-    @http.request_get(@parsed_uri.request_uri, 'Accept-Encoding' => 'identity') do |res|
+    @http.request_get(@parsed_uri.request_uri, http_header) do |res|
       if res.is_a?(Net::HTTPRedirection) && @redirect_count < 4
         @redirect_count += 1
         begin
@@ -231,6 +247,8 @@ class FastImage
 
       raise ImageFetchFailure unless res.is_a?(Net::HTTPSuccess)
 
+      @content_length = res.content_length
+
       read_fiber = Fiber.new do
         res.read_body do |str|
           Fiber.yield str
@@ -260,8 +278,8 @@ class FastImage
 
   def proxy_uri
     begin
-      if @proxy_url
-        proxy = Addressable::URI.parse(@proxy_url)
+      if @options[:proxy]
+        proxy = Addressable::URI.parse(@options[:proxy])
       else
         proxy = ENV['http_proxy'] && ENV['http_proxy'] != "" ? Addressable::URI.parse(ENV['http_proxy']) : nil
       end
@@ -281,8 +299,8 @@ class FastImage
     end
     @http.use_ssl = (@parsed_uri.scheme == "https")
     @http.verify_mode = OpenSSL::SSL::VERIFY_NONE
-    @http.open_timeout = @timeout
-    @http.read_timeout = @timeout
+    @http.open_timeout = @options[:timeout]
+    @http.read_timeout = @options[:timeout]
   end
 
   def fetch_using_read(readable)
@@ -343,6 +361,14 @@ class FastImage
     def read_int
       read(2).unpack('n')[0]
     end
+
+    def read_string_int
+      value = []
+      while read(1) =~ /(\d)/
+        value << $1
+      end
+      value.join.to_i
+    end
   end
 
   class FiberStream # :nodoc:
@@ -410,6 +436,14 @@ class FastImage
       else
         raise UnknownImageType
       end
+    when "<s"
+      :svg
+    when "<?"
+      if @stream.peek(100).include?("<svg")
+        :svg
+      else
+        raise UnknownImageType
+      end
     else
       raise UnknownImageType
     end
@@ -596,4 +630,68 @@ class FastImage
   def parse_size_for_psd
     @stream.read(26).unpack("x14NN").reverse
   end
+
+  class Svg # :nodoc:
+    def initialize(stream)
+      @stream = stream
+      parse_svg
+    end
+
+    def width_and_height
+      if @width && @height
+        [@width, @height]
+      elsif @width && @ratio
+        [@width, @width / @ratio]
+      elsif @height && @ratio
+        [@height * @ratio, @height]
+      end
+    end
+
+    private
+
+    def parse_svg
+      attr_name = []
+      state = nil
+
+      while (char = @stream.read(1)) && state != :stop do
+        case char
+        when "="
+          if attr_name.join =~ /width/i
+            @stream.read(1)
+            @width = @stream.read_string_int
+            return if @height
+          elsif attr_name.join =~ /height/i
+            @stream.read(1)
+            @height = @stream.read_string_int
+            return if @width
+          elsif attr_name.join =~ /viewbox/i
+            values = attr_value.split(/\s/)
+            @ratio = values[2].to_f / values[3].to_f
+          end
+        when /\w/
+          attr_name << char
+        when ">"
+          state = :stop if state == :started
+        else
+          state = :started if attr_name.join == "svg"
+          attr_name.clear
+        end
+      end
+    end
+
+    def attr_value
+      @stream.read(1)
+
+      value = []
+      while @stream.read(1) =~ /([^"])/
+        value << $1
+      end
+      value.join
+    end
+  end
+
+  def parse_size_for_svg
+    svg = Svg.new(@stream)
+    svg.width_and_height
+  end
 end
