@@ -30,12 +30,15 @@ GoodFixtures = {
   "test.cur" => [:cur, [32, 32]],
   "webp_vp8x.webp" => [:webp, [386, 395]],
   "webp_vp8l.webp" => [:webp, [386, 395]],
-  "webp_vp8.webp" => [:webp, [550, 368]]
+  "webp_vp8.webp" => [:webp, [550, 368]],
+  "test.svg" => [:svg, [200, 300]],
+  "test_partial_viewport.svg" => [:svg, [860, 400]]
 }
 
 BadFixtures = [
   "faulty.jpg",
-  "test_rgb.ct"
+  "test_rgb.ct",
+  "test.xml"
 ]
 # man.ico courtesy of http://www.iconseeker.com/search-icon/artists-valley-sample/business-man-blue.html
 # test_rgb.ct courtesy of http://fileformats.archiveteam.org/wiki/Scitex_CT
@@ -120,6 +123,12 @@ class FastImageTest < Test::Unit::TestCase
     end
   end
 
+  def test_should_raise_unknown_image_typ_when_file_is_non_svg_xml
+    assert_raises(FastImage::UnknownImageType) do
+      FastImage.size(TestUrl + "test.xml", :raise_on_failure=>true)
+    end
+  end
+
   def test_should_report_type_correctly_for_local_files
     GoodFixtures.each do |fn, info|
       assert_equal info[0], FastImage.type(File.join(FixturePath, fn))
@@ -297,4 +306,18 @@ class FastImageTest < Test::Unit::TestCase
   ensure
     %x{rm -f shell_test}
   end
+
+  def test_content_length
+    url = "#{TestUrl}with_content_length.gif"
+    FakeWeb.register_uri(:get, url, :body => File.join(FixturePath, "test.jpg"), :content_length => 52)
+
+    assert_equal 52, FastImage.new(url).content_length
+  end
+
+  def test_content_length_not_provided
+    url = "#{TestUrl}without_content_length.gif"
+    FakeWeb.register_uri(:get, url, :body => File.join(FixturePath, "test.jpg"))
+
+    assert_equal nil, FastImage.new(url).content_length
+  end
 end
